# 
# The program was written or based on one written in 2004 by Morgan Ames.
# http://www.ocf.berkeley.edu/~morganya
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the Creative Commons Attribution-NonCommercial 
# License. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE (as described by
# the GNU General Public License at http://www.gnu.org/licenses/gpl.html). 
#
# To view a copy of the Creative Commons Attribution-NonCommercial license, 
# visit http://creativecommons.org/licenses/by-nc/2.0/ or send a letter to 
# Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.

'''
TODO:

redo photo pages: divide into smaller picture groups
delete pictures with a web interface
make sure there aren't any weird characters in directories or filenames
automatically generate favorite-photos?

'''

import os, time, glob

debugging = 1

title_file = "titles.txt"
header_file = "header.txt"
footer_file = "footer.txt"
root_path = "../"
pic_height = 600
thumb_prefix = "T"
thumb_height = 200
hid = 'hid-'
thumb = 'thumbnail-'
favorite = 'favorite-'
delete = 'delete-'
photo_path = '/~morganya/photos/'
no_thumbnail = 'No thumbnail'
reserved_names = ['thumbnail.jpg', 'manage', 'favorite-photos.html']

quote = '''
<h4>"A good snapshot stops a moment from running away."<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; - Eudora Welty</h4>
'''


def get_quote():
    return quote

def get_header():
    file = open(header_file)
    header_string = file.read()
    file.close()
    return header_string

def get_footer():
    file = open(footer_file)
    footer_string = file.read()
    file.close()
    return footer_string

def chmod(mode, path):
    for file in glob.glob(path):
        os.chmod(file, mode)

def parse_titles(filename=title_file):
    file = open(filename)
    title_list = file.read().replace('\r', '').split('\n')
    file.close()
    titles = {}
    for line in title_list:
        if line:
            (location, thumbnail, title) = line.split(',', 2)
            location = location.strip('/')
            titles[location] = {'thumbnail': thumbnail.strip(),
                                'title': title.strip()}
    return titles


def write_titles(titles, filename=title_file):
    title_string = ''
    keys = titles.keys()
    keys.sort()
    for location in keys:
        title_string += ','.join([location, \
                                  titles[location]['thumbnail'], \
                                  titles[location]['title']]) + '\n'
    file = open(filename, 'w')
    file.write(title_string)
    file.close()


def listdir(directory):
    list = os.listdir(directory)
    for name in reserved_names:
        if name in list:
            list.remove(name)
    list.sort()
    return list


def fix_names(directory):
    '''Change characters in the directory and file names as follows:
       space ( ) -> underscore (_)
       apostrophe (') -> none
       quote (") -> none
    '''
    pass # TODO


def resize(directory):
    files = listdir(root_path + directory)
    for file in files:
        if file[-3:].lower() == 'jpg' and \
           file not in reserved_names and \
           not file.startswith(thumb_prefix):
            pic_path = root_path + directory + '/' + file
            if debugging:
                print "Resizing " + pic_path + ' ... '
            status = os.system("/opt/local/bin/djpeg " + pic_path + \
                               " | /opt/local/bin/pnmscale -height " + str(pic_height) + \
                               " | /opt/local/bin/cjpeg > ./temp.jpg")
            pic_size = os.stat(pic_path)[6]
            temp_size = os.stat('./temp.jpg')[6]
            if temp_size < pic_size:
                if debugging:
                    print file, '- old size =', pic_size/1024.0, \
                          'KB, and new size =', temp_size/1024.0, 'KB'
                os.system('/usr/bin/mv ./temp.jpg ' + pic_path)
            if debugging:
                print "Finished resizing " + pic_path + \
                      ', exit status ' + str(status) + '.<br>\n'
    chmod(0744, root_path + directory + "/*")


def make_thumbnails(directory):
    files = listdir(root_path + directory)
    for file in files:
        if file[-3:].lower() == 'jpg' and \
           file not in reserved_names and \
           not file.startswith(thumb_prefix) and \
           thumb_prefix + file.lower() not in files:
           # TODO make sure not to make thumbnails of thumbnails in a better way than this - 
           # if possible, add check of height
            if debugging:
                print "Making thumbnail of " + root_path + directory + '/' + file + ' ... '
            status = os.system("/opt/local/bin/djpeg " + root_path + directory + '/' + file + \
                               " | /opt/local/bin/pnmscale -height " + str(thumb_height) + \
                               " | /opt/local/bin/cjpeg > " + root_path + directory + '/' + thumb_prefix + file.lower())
            if debugging:
                print "Finished making thumbnail " + \
                      root_path + directory + '/' + thumb_prefix + file.lower() + \
                      ', exit status ' + str(status) + '.<br>\n'
    chmod(0744, root_path + directory + "/*")


def rename_photos(directory, start):
    files = listdir(root_path + directory)
    titles = parse_titles()
    changed = 0
    prefix = '%0' + str(max((len(str(len(get_jpg_files(files, 0))))-1), 2)) + 'd'
    for file in files:
        (name, extenstion) = file.split('.')
        if extension.lower() == 'jpg' and \
           file not in reserved_names and \
           not name.startswith(thumb_prefix):
            new_name = prefix % str(start) + '.' + extension.lower()
            os.rename(root_path + directory + '/' + file,
                      root_path + directory + '/' + new_name)
            if thumb_prefix + file in files:
                os.rename(root_path + directory + '/' + thumb_prefix + file,
                          root_path + directory + '/' + thumb_prefix + new_name)
            if not changed and name == titles[directory]['thumbnail']:
                titles[directory]['thumbnail'] = new_name
                changed = 1
            start += 1
    write_titles(titles)
    

def get_jpg_files(list, thumb=1):
    return_list = []
    for item in list:
        if item[-3:].lower() == 'jpg':
            if not item.startswith(thumb_prefix) or thumb == 1:
                return_list.append(item)
    return return_list


def photo_html(file, files, directory, main_index):
    ''' Returns the text for an html file for individual photos.
    '''
    html_string = ''
    prev = ''
    next = ''
    album_title = parse_titles().get(directory, {}).get('title', '')
    year = main_index.split('.')[0]
    if year == 'index':
        year = str(time.localtime()[0])
    files = get_jpg_files(files, thumb=0)
    index = files.index(file)
    if index > 0:
        prev = '<a href="' + photo_link(files[index-1].split('.')[0]) + '">&lt;&lt; Previous</a>'
    if files.index(file) < len(files)-1:
        next = '<a href="' + photo_link(files[index+1].split('.')[0]) + '">Next &gt;&gt;</a>'
    html_string += get_header() + '''
<table>
<tr>
  <td colspan=4>
    <a href="../''' + main_index + '">' + year + '</a>: <a href="index.html">' + album_title + '''</a><br>
    Photo ''' + str(index+1) + ' of ' + str(len(files)) + '''</td>
</tr>
<tr>
  <td align="left" width="200">''' + prev + '''</td>
  <td align="center" width="200"><a href="index.html#''' + file.split('.')[0] + '''">Back</a></td>
  <td align="right" width="200">''' + next + '''</td>
  <td></td>
</tr>
<tr>
  <td colspan=4>
    <img src="''' + file + '''">
  </td>
</tr>
<tr>
  <td align="left" width="200">''' + prev + '''</td>
  <td align="center" width="200"><a href="index.html#''' + file.split('.')[0] + '''">Back</a></td>
  <td align="right" width="200">''' + next + '''</td>
  <td></td>
</tr>
</table>''' + get_footer() 
    return html_string


def photo_link(photo_name):
    return photo_name + '.html'


def write_photo_wrappers(directory, main_index): 
    files = listdir(root_path + directory)
    for file in files:
        (name, extension) = file.split('.')
        if extension.lower() == 'jpg' and \
           file not in reserved_names and \
           not file.startswith(thumb_prefix):
            html = photo_html(file, files, directory, main_index)
            file = open(root_path + directory + '/' + name + '.html', 'w')
            file.write(html)
            file.close()
    index_html = get_header() + write_file_html(directory, main_index=main_index) + get_footer()
    file = open(root_path + directory + '/index.html', 'w')
    file.write(index_html)
    file.close()
    chmod(0744, root_path + directory + "/*")


def write_index(year):
    '''Write year album summaries, including each directory in that year.
    '''
    files = listdir(root_path)
    titles = parse_titles()

    file_to_write = str(year) + '.html'
    if year == time.localtime()[0]:
        file_to_write = 'index.html'

    year_suffix = str(year)[-2:]
    html_string = ''

    picture_count = 0
    album_count = 0
    for file in files:
        if file not in reserved_names and '.' not in file and (file.startswith(year_suffix) or \
           (year_suffix == '99' and int(file[:2]) > 10 and int(year_suffix) > int(file[:2]))):
            album_count = count_pictures(listdir(root_path + file))
            html_string += '<a href="' + file + '">'
            html_string += titles.get(file, {}).get('title', file) + \
                           ' (' + str(album_count) + pluralize(album_count, ' photo') + ')<br>'
            if titles.get(file, {}).get('thumbnail'):
                html_string += '<img src="' + file + '/' + thumb_prefix + \
                               titles[file]['thumbnail'] + '"><br>'
            html_string += '</a><br>'
            album_count += 1
            picture_count += album_count
    html_string += '<a href="manage">Manage photos ...</a><p>' + get_footer()

    name = file_to_write.split('.')[0]
    if name == 'index':
        name = str(time.localtime()[0])

    html_prefix = get_header() + get_quote() + '<h3>' + name + ' (' + \
                  str(picture_count) + pluralize(picture_count, ' photo') + \
                  ' in ' + str(album_count) + pluralize(album_count, ' album') + ')</h3>' + \
                  '<h4>Other years: '

    year_list = []
    for file in files:
        name = file.split('.')[0]
        if name == 'index':
            name = str(time.localtime()[0])
        if file[-4:] == 'html':
            if file == file_to_write:
                year_list.append(name)
            else:
                year_list.append('<a href="' + file + '">' + name + '</a>')
    html_prefix += ', '.join(year_list) + '</h4>'
    
    html_string = html_prefix + html_string
    
    if debugging:
        print 'Writing file ' + file_to_write + '<br>\n'
        
    file = open(root_path + file_to_write, 'w')
    file.write(html_string)
    file.close()
    chmod(0755, root_path + "*")


def pluralize(count, str):
    if count == 1:
        return str
    else:
        return str + 's'


def write_file_html(directory, path=photo_path, main_index='index.html', editing=0):
    titles = parse_titles()
    pics = listdir(root_path + directory)
    picture_count = count_pictures(pics)
    year = main_index.split('.')[0]
    if year == 'index':
        year = str(time.localtime()[0])

    html_string = '<a href="' + path + main_index + '">' + year + '</a>: ' + \
                  titles.get(directory, {}).get('title', '') + '<br>\n'        
    html_string += ' (' + str(picture_count) + pluralize(picture_count, ' photo') + ')<br><br>\n'
    if editing:
        checked = 'CHECKED'
        if titles.get(directory, {}).get('thumbnail', '') in pics:
            checked = ''
        html_string += '<input type="radio" name="' + thumb + directory + \
                       '" value="' + no_thumbnail + '" ' + checked + '> No album thumbnail<br><br>\n'
        full_path = path + directory + '/'
    else:
        # TODO link back to year album
        full_path = ''
    for pic in pics:
        (name, extension) = pic.split('.')
        if extension == 'html' and (name + '.jpg' in pics or name + '.JPG' in pics):
            html_string += '<a name="' + name + '"></a>' + \
                           '<a href="' + full_path + pic + '">' + \
                            '<img src="' + full_path + thumb_prefix + name.lower() + '.jpg"></a>\n'
            if editing:
                checked = ''
                if titles.get(directory, {}).get('thumbnail', '').lower() == (name.lower()+'.jpg'):
                    checked = 'CHECKED'
                html_string += '<br><input type="radio" name="' + thumb + directory + \
                               '" value="' + (name.lower()+'.jpg') + '" ' + checked + '> Album thumbnail\n'
                # TODO html_string += '<br><input type="checkbox" name="' + delete + directory + \
                #               '" value="' + pic + '" ' + checked + '> Delete\n'
                # TODO html_string += '<input type="checkbox" name="' + favorite + directory + '" value="' + pic + '"> Favorite picture<br>\n'
                html_string += '<br><br>'
    html_string += '<br>\n<br>\n\n'
    return html_string


def count_pictures(pics):
    return len(get_jpg_files(pics, thumb=0))


def main():
    files = listdir(root_path)
    for file in files:
        if file not in reserved_names and \
           '.' not in file:
            if debugging:
                print file + '<br>\n'
            yr = file[:2]            
            if int(yr) > 10:
                main_index = '1999.html'
            elif yr == str(time.localtime()[0])[-2:]:
                main_index = 'index.html'
            else:
                main_index = '20' + yr + '.html'
            write_photo_wrappers(file, main_index)

    for year in range(1999, time.localtime()[0]+1):
        write_index(year)


def edit_favorites():
    # TODO
    pass


def edit_titles():
    '''Return html page for editing titles and thumbnails of picture albums.
    '''
    titles = parse_titles()
    files = listdir(root_path)
    files.reverse()
    html_string = get_header() + '''
<form action="save_titles.cgi" method="post">
<h4>Edit titles and thumbnails</h4>

'''
    for file in files:
        if file not in reserved_names and \
           '.' not in file:
            
            html_string += file + ': <br>' + \
                           '<input type="text" size="40" name="' + file + '" value="' + \
                           html_escape(titles.get(file, {}).get('title', '')) + '"><br>\n' + \
                           '<input type="hidden" name="' + hid + file + '" value="' + \
                           html_escape(titles.get(file, {}).get('title', '')) + '"><br>\n' + \
                           write_file_html(file, editing=1)
    html_string += '''
<input type="submit" name="Submit changes">
</form>''' + get_footer()
    return html_string


def save_titles(dict):
    titles = parse_titles()
    for key in dict.keys():
        if key.startswith(thumb):
                dir = key.lstrip(thumb)
                if dir not in titles:
                    titles[dir] = {'thumbnail': '', 'title': ''}
                if dict[key] == no_thumbnail:
                    titles[dir]['thumbnail'] = ''
                else:
                    titles[dir]['thumbnail'] = dict[key]
                if debugging:
                    print 'Setting thumbnail of ' + dir + ' to ' + dict[key] + '.<br>\n'
        elif not key.startswith(hid):
            new = html_escape(dict.get(key, ''), 1)
            old = html_escape(dict.get(hid + key, ''), 1)
            if not new == old:
                if key not in titles:
                    titles[key] = {'thumbnail': '', 'title': ''}
                titles[key]['title'] = new
                if debugging:
                    print 'Setting title of ' + key + ' to ' + new + '.<br>\n'
        elif key[len(hid):] not in dict:
            key1 = key[len(hid):]
            if key not in titles:
                titles[key1] = {'thumbnail': '', 'title': ''}
            titles[key1]['title'] = ''
            if debugging:
                print 'Deleting title of ' + key1 + '.<br>\n'
    write_titles(titles)



def html_escape(string, reverse=0):
    if reverse:
        return string.replace('&lt;', '<').replace('&gt;', '>').replace('&quot;', '"')
    else:
        return string.replace('\r', '').replace('\n', '').replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;')



def index():
    files = listdir(root_path)
    html_string =  get_header() + get_quote() + '''

<form action="edit_titles.cgi" method="post">
<input type="submit" value="Edit titles">
</form>

<form action="switch.cgi" method="post">
<table><tr><td valign="top">

<b>Actions:</b><br>
  <input type="checkbox" name="make_thumbnails" value="selected">Make thumbnails<br>
  <input type="checkbox" name="resize_pics" value="selected">Shrink pictures<br>
  <input type="checkbox" name="rewrite_html" value="selected" checked>Rewrite HTML<br>

<input type="submit" value="Submit"><br>

</td><td valign="top">

<b>Directories:</b><br>'''
    for file in files:
        if file not in reserved_names and \
           '.' not in file:
            html_string += '  <input type="checkbox" name="directory" value="' + file + '">' + file + '<br>\n'
    html_string += '''
</td></tr></table>
</form>
'''

    html_string +=  get_footer()
    return html_string
